// ===========================================================================
//	LGEPane.h					1994 by Al Evans. All rights reserved.
//
//	A Metrowerks PowerPlant Pane for Graphic Elements GEWorlds
//
//	This pane is somewhat rudimentary, but functional. In particular:
//
//	- It has no support for GEWorld scales other than scaleOneToOne
//	- It determines the size of the GEWorld to create by its own size
//	- The procedure for deleting and recreating the pane's GEWorld should be 
//	  more orthogonal
//
//	It does support the use of a PICT as a replacement for a temporarily-
//	deleted GEWorld.
//
// ===========================================================================

#pragma once

#include <LPane.h>
#include <LPeriodical.h>
#include "GraphElements.h"

// Class ID for use in Constructor
const ClassIDT	gePaneType	= 'GEpn';


// Typedef of function to load a GEWorld with Graphic Elements
typedef Boolean (*GEWorldLoader)(GEWorldPtr world);

class	LGEPane :	public LPane,
					public LPeriodical {
public:
	static LGEPane*	CreateLGEPaneStream(LStream *inStream);
	
						LGEPane(LStream *inStream);
						LGEPane();
						LGEPane(const SPaneInfo &inPaneInfo,
									GEWorldLoader loadProc = nil);
	virtual				~LGEPane();
	
	virtual Boolean		MakeWorld();
	virtual void		DisposeWorld();
	virtual	Boolean		LoadWorld(GEWorldLoader loadProc);
	void				SetSubstitutePICT(ResIDT pictResNum);
	GEWorldPtr			GetGEWorld();			
	
// LPeriodical
	
	virtual void		SpendTime(const EventRecord &inMacEvent);


protected:
	GEWorldPtr			mGEWorld;
	ResIDT				mSubstitutePICT;
	Boolean				mLoaded;
	
	virtual void		DrawSelf();
	virtual void		ClickSelf(const SMouseDownEvent &inMouseDown);
};
