/*
	File:		PPColorPicker.cp

	Contains:	Template code for molding to your own app's
				desires.  PowerPlant style.
			
	Written by: Chris Thomas

	Copyright:	 1995 Chris K. Thomas. All Rights Reserved.

	Change History (most recent first):
	 			8/26/95		ckt		Created
	 
	 We use an LAttachment to get events from the current
	 LEventDispatcher and send them to us unmodified.
*/

#include "PPColorPicker.h"
#include "LColorPickerEvents.h"

#include "UAttachments.h"
#include <LEventDispatcher.h>
#include <LMenuBar.h>
#include <LView.h>

// ѥConstructor
//   build self

PPColorPicker::PPColorPicker(LPane *inPane, StringPtr inPrompt, RGBColor& inOldColor,
											Boolean isMovable, Boolean isModal)
:LColorPicker(inPrompt, inOldColor, isMovable, isModal)
{
	LView::OutOfFocus(NULL);
	
	mPane = inPane;
	mDispatcher = NULL;
	mEventsAttachment = NULL;

	mDispatcher = LEventDispatcher::GetCurrentEventDispatcher();
	ThrowIfNULL_(mDispatcher);	// * must have events!
	
	mEventsAttachment = new LColorPickerEvents(this);
	ThrowIfNULL_(mEventsAttachment);
	
	mDispatcher->AddAttachment(mEventsAttachment, nil, false);
	
	AddUser(this);
}

// ѥDestructor
//   kill self
PPColorPicker::~PPColorPicker()
{
	if(mDispatcher != NULL && mEventsAttachment != NULL)
	{
		mDispatcher->RemoveAttachment(mEventsAttachment);
	}
}

// ѥGetMenuItems
//   fill in menu item info
void PPColorPicker::GetMenuItems(PickerMenuItemInfo &outInfo)
{
	// * In PowerPlant, we can grab the edit menu indexes automatically
	LMenuBar	*curMenuBar = LMenuBar::GetCurrentMenuBar();
	if(curMenuBar)
	{
		MenuHandle	menuH = NULL;
		
		curMenuBar->FindMenuItem(cmd_Undo, outInfo.editMenuID, menuH, outInfo.undoItem);
		curMenuBar->FindMenuItem(cmd_Cut, outInfo.editMenuID, menuH, outInfo.cutItem);
		curMenuBar->FindMenuItem(cmd_Copy, outInfo.editMenuID, menuH, outInfo.copyItem);
		curMenuBar->FindMenuItem(cmd_Paste, outInfo.editMenuID, menuH, outInfo.pasteItem);
		curMenuBar->FindMenuItem(cmd_Clear, outInfo.editMenuID, menuH, outInfo.clearItem);
	}
	else
		throw 'nilp';
}

// ѥUserChangingColor
//   called on the fly, while the user is selecting new colors.
//     don't save the color permanently, as the user could cancel it.
void PPColorPicker::UserChangingColor()
{
	RGBColor	currentColor;
	
	GetColor(currentColor);
	
	// * specific to sample app
	PenState			currentState;
	GetPenState(&currentState);
	
	LPaintAttachment	ourPaint(&currentState, &currentColor, nil, false);
	
	mPane->FocusDraw();
	mPane->AddAttachment(&ourPaint);
	mPane->Draw(NULL);
	mPane->RemoveAttachment(&ourPaint);
	
	// ...
}

// ѥUserFinalizedColor
//   called when the user has picked a color
//     you still might not save this permanently as the user might click cancel
void PPColorPicker::UserFinalizedColor()
{
	RGBColor	currentColor;
	
	GetColor(currentColor);
	
	// ...
}

// ѥUserSaysOK
//   user clicked OK
void PPColorPicker::UserSaysOK()
{
	// remove your app's reference(s) to this class
	RemoveUser(this);
}


// ѥUserSaysCancel
//   user clicked Cancel
void PPColorPicker::UserSaysCancel()
{
	// remove your app's reference(s) to this class
	RemoveUser(this);
}