unit MySystemGlobals;

interface

	uses
		Quickdraw, Files;

	var  { set up by InitSystemGlobals }
		system7: boolean;
		system_version: longint;
		has_appleEvents: boolean;
		has_findfolder: boolean;
		has_aliasMgr: boolean;
		has_newStdFile: boolean;
		has_HelpMgr: boolean;
		has_colorQD: boolean;
		has_LaunchControl: Boolean;
		app_resfile: integer;
		app_fs: FSSpec;
		in_foreground: boolean;

	procedure InitSystemGlobals;
	function RefNumToFSSpec(rn:integer; var fs:FSSpec):OSErr;
	procedure SetInForeground (fore: boolean);
	function InForeground: boolean;
	
implementation

	uses
		Resources, GestaltEqu, Processes;
		
	function TPbtst(value:longint; bit:integer):Boolean;
	begin
		TPbtst := btst(value, bit);
	end;
	
	procedure SetInForeground (fore: boolean);
	begin
		in_foreground := fore;
	end;

	function InForeground: boolean;
		var
			gv: longint;
			ourpsn, frontpsn: ProcessSerialNumber;
			front: boolean;
	begin
		if (Gestalt(gestaltOSAttr, gv) = noErr) & (TPbtst(gv, gestaltLaunchControl)) then begin
			if (GetCurrentProcess(ourpsn) = noErr) & (GetFrontProcess(frontpsn) = noErr) then begin
				if SameProcess(ourpsn, frontpsn, front) = noErr then begin
					in_foreground := front;
				end;
			end;
		end;
		InForeground := in_foreground;
	end;

	function RefNumToFSSpec(rn:integer; var fs:FSSpec):OSErr;
		var
			pb: FCBPBRec;
	begin
		pb.ioNamePtr := @fs.name;
		pb.ioVRefNum := 0;
		pb.ioRefNum := rn;
		pb.ioFCBIndx := 0;
		RefNumToFSSpec := PBGetFCBInfoSync(@pb);
		fs.vRefNum := pb.ioFCBVRefNum;
		fs.parID := pb.ioFCBParID;
	end;
	
	procedure InitSystemGlobals;
		var
			oe: OSErr;
			gv: longint;
			sysenv: sysEnvRec;
	begin
		app_resfile := CurResFile;
		oe := RefNumToFSSpec(app_resfile, app_fs);
		has_colorqd := (SysEnvirons(1, sysEnv) = noErr) & sysenv.hasColorQD; { Gestalt has a bug that causes hasColourQD to always be set }
		if (Gestalt(gestaltSystemVersion, system_version) <> noErr) then begin
			system_version := $0600;
		end;
		system7 := system_version >= $0700;
		has_AppleEvents := (Gestalt(gestaltAppleEventsAttr, gv) = noErr) & (TPbtst(gv, gestaltAppleEventsPresent));
		has_findfolder := (Gestalt(gestaltFindFolderAttr, gv) = noErr) & (TPbtst(gv, gestaltFindFolderPresent));
		has_newStdFile := (Gestalt(gestaltStandardFileAttr, gv) = noErr) & (TPbtst(gv, gestaltStandardFile58));
		has_HelpMgr := (Gestalt(gestaltHelpMgrAttr, gv) = noErr) & (TPbtst(gv, gestaltHelpMgrPresent));
		has_aliasMgr := (Gestalt(gestaltAliasMgrAttr, gv) = noErr) & (TPbtst(gv, gestaltAliasMgrPresent));
		has_LaunchControl := (Gestalt(gestaltOSAttr, gv) = noErr) & (TPbtst(gv, gestaltLaunchControl));
		in_foreground := true;
	end;

end.