/********************						***********************/
//
//	Player PRO 4.4x -- S3M to MADx
//
//	Version 1.0	- 12.3.95 ANR
//
//	To use with CodeWarrior 68K or PPC
//
//	Antoine ROSSET
//	16 Tranchees
//	1206 GENEVA
//	SWITZERLAND
//	
//	FAX:			(+41 22) 346 11 97
//	Compuserve:	100277,164
//	Internet: 		rosset@dial.eunet.ch
//
/********************						***********************/

#include "S3M.h"
#include "MAD.h"
#include "RDriver.h"

#if defined(powerc) || defined(__powerc)
enum {
		PlayerPROPlug = kCStackBased
		| RESULT_SIZE(SIZE_CODE(sizeof(OSErr)))
		| STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof( OSType)))
		| STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof( Ptr*)))
		| STACK_ROUTINE_PARAMETER(3, SIZE_CODE(sizeof( MADPartition*)))
		| STACK_ROUTINE_PARAMETER(4, SIZE_CODE(sizeof( PPInfoRec*)))
};

ProcInfoType __procinfo = PlayerPROPlug;
#else
#include <A4Stuff.h>
#endif

#define LOW(para) ((para) & 15)
#define HI(para) ((para) >> 4)

unsigned long Tdecode32( void *msg_buf)
{
  unsigned char *buf = msg_buf;
  
  return( (unsigned long) buf[3] << 24) | ( (unsigned long) buf[2] << 16) | ( (unsigned long) buf[ 1] << 8) | ( (unsigned long) buf[0]);
}

short Tdecode16( void *msg_buf)
{
  unsigned char *buf = msg_buf;
  
  return ( (short) buf[1] << 8) | ( (short) buf[0]);
}

Cmd* GetMADCommand( register short PosX, register short	TrackIdX, register PatData*	tempMusicPat)
{
	if( PosX < 0) PosX = 0;
	else if( PosX >= tempMusicPat->header.size) PosX = tempMusicPat->header.size -1;
		
	return( & (tempMusicPat->Cmds[ (tempMusicPat->header.size * TrackIdX) + PosX]));
}

void pStrcpy(register unsigned char *s1, register unsigned char *s2)
{
	register short len, i;
	
	len = *s2;
	for ( i = 0; i <= len; i++) s1[ i] = s2[ i];
}

void ConvertS3MEffect( Byte B0, Byte B1, Byte *Cmd, Byte *Arg)
{
	Str255	tStr;
	Byte		LoB1 = LOW( B1);
	Byte		HiB1 = HI( B1);
	
	switch( B0 + 0x40)
	{
		default:	*Cmd = 0;			*Arg = 0;		break;
		// Speed
		case 'A':	*Cmd = speedE;		*Arg = B1;	break;
		// Tempo
		case 'T':	*Cmd = speedE;		*Arg = B1;	break;

		case 'B':	*Cmd = fastskipE;		*Arg = B1;	break;

		case 'C':	*Cmd = skipE;			*Arg = B1;	break;

		case 'D':
			if( LoB1 == 0 || HiB1 == 0)		// Slide volume
			{
				*Cmd = slidevolE;		*Arg = B1;
			}
			else if( HiB1 == 0x0F)		// Fine Slide volume DOWN
			{
				*Cmd = extendedE;
				*Arg = 11 << 4;
				*Arg += LoB1;
			}
			else if( LoB1 == 0x0F)		// Fine Slide volume UP
			{
				*Cmd = extendedE;
				*Arg = 10 << 4;
				*Arg += HiB1;
			}
		break;
		
		case 'E':
			if( HiB1 == 0x0F)		// FineSlide DOWN
			{
				*Cmd = extendedE;
				*Arg = 2 << 4;		//not supported
				*Arg += LoB1;
			}
			else if( HiB1 == 0x0E)	// ExtraFineSlide DOWN
			{
				*Cmd = 0;			*Arg = 0;		//not supported
			}
			else					// Slide DOWN
			{
				*Cmd = upslideE;		*Arg = B1;
			}
		break;

		case 'F':
			if( HiB1 == 0x0F)		// FineSlide UP
			{
				*Cmd = extendedE;
				*Arg = 1 << 4;		//not supported
				*Arg += LoB1;
			}
			else if( HiB1 == 0x0E)	// ExtraFineSlide UP
			{
				*Cmd = 0;			*Arg = 0;		//not supported
			}
			else					// Slide UP
			{
				*Cmd = downslideE;		*Arg = B1;
			}
		break;

		case 'G':	*Cmd = portamentoE;	*Arg = B1;	break;
		case 'H':	*Cmd = vibratoE;		*Arg = B1;	break;
		
		case 'J':	*Cmd = arpeggioE;		*Arg = B1;	break;
		case 'K':	*Cmd = vibratoslideE;	*Arg = B1;	break;
		case 'L':	*Cmd = portaslideE;		*Arg = B1;	break;
		case 'O':	*Cmd = offsetE;		*Arg = B1;	break;

		case 'S':		// Special Effects
			switch( HiB1)
			{
				default:	*Cmd = 0;		*Arg = 0;							break;
				case 2:	*Cmd = extendedE;	*Arg = 5 << 4;		*Arg += LoB1;		break;	// FineTune
				case 3:	*Cmd = extendedE;	*Arg = 4 << 4;		*Arg += LoB1;		break;	// Set Vibrato WaveForm
				case 4:	*Cmd = extendedE;	*Arg = 7 << 4;		*Arg += LoB1;		break;	// Set Tremolo WaveForm
				case 0xB:	*Cmd = extendedE;	*Arg = 6 << 4;		*Arg += LoB1;		break;	// Loop pattern
				case 0xC:	*Cmd = extendedE;	*Arg = 12 << 4;	*Arg += LoB1;		break;	// Cut sample
				case 0xD:	*Cmd = extendedE;	*Arg = 13 << 4;	*Arg += LoB1;		break;	// Delay sample
				case 0xE:	*Cmd = extendedE;	*Arg = 14 << 4;	*Arg += LoB1;		break;	// Delay pattern
			}
		break;
	}
}

OSErr ConvertS3M2Mad( Ptr	theS3M, long MODSize, MADPartition *theMAD)
{
	long 				i, PatMax, x, z, channel, Row;
	long 				sndSize, OffSetToSample, OldTicks, temp, starting;
	Ptr				MaxPtr;
	OSErr			theErr;
	Ptr				theInstrument[ 64], destPtr;
	Byte				tempChar, *theS3MCopy;
	short			Note, Octave, maxTrack;
	short			S3Mperiod[ 12] = {1712,1616,1524,1440,1356,1280,1208,1140,1076,1016, 960, 907};
	long				note_st3period;
	long				note_amigaperiod;
	Byte				*ChannelSettings;
	
	/**** Variables pour le MAD ****/
	Cmd				*aCmd;

	/**** Variables pour le S3M ****/
	
	s3mform			s3minfo;
	/********************************/

	for( i = 0 ; i < 64; i ++)
	{
		theInstrument[ i] = 0L;
	}

	/**** Header principal *****/
	theS3MCopy = (Byte*) theS3M;
	BlockMove( theS3MCopy, &s3minfo, 96);
	theS3MCopy += 96;
	
	s3minfo.ordernum	= Tdecode16(  &s3minfo.ordernum);
	s3minfo.insnum		= Tdecode16(  &s3minfo.insnum);
	s3minfo.patnum		= Tdecode16(  &s3minfo.patnum);
	s3minfo.flags		= Tdecode16(  &s3minfo.flags);
	s3minfo.cwtv		= Tdecode16(  &s3minfo.cwtv);
	s3minfo.ffv		= Tdecode16(  &s3minfo.ffv);

	/**** Order Num *****/
	s3minfo.orders = (unsigned char *) NewPtr( s3minfo.ordernum);
	BlockMove( theS3MCopy, s3minfo.orders, s3minfo.ordernum);
	theS3MCopy += s3minfo.ordernum;
	
	/**** Ins Num *****/
	s3minfo.parapins = (unsigned short *) NewPtr( s3minfo.insnum * 2L);
	BlockMove( theS3MCopy, s3minfo.parapins, s3minfo.insnum * 2L);
	theS3MCopy += s3minfo.insnum * 2L;
	for( i = 0; i < s3minfo.insnum; i++)
	{
		s3minfo.parapins[ i] = Tdecode16(  &s3minfo.parapins[ i]);
	}
	
	
	/**** Pat Num *****/
	s3minfo.parappat = (unsigned short *) NewPtr( s3minfo.patnum * 2L);
	BlockMove( theS3MCopy, s3minfo.parappat, s3minfo.patnum * 2L);
	theS3MCopy += s3minfo.patnum * 2L;
	for( i = 0; i < s3minfo.patnum; i++)
	{
		s3minfo.parappat[ i] = Tdecode16(  &s3minfo.parappat[ i]);
	}

	/**** Ins Data ****/
	if( s3minfo.insnum > 64) s3minfo.insnum = 64;
	s3minfo.insdata = (s3minsform *) NewPtr( sizeof(s3minsform) * s3minfo.insnum);
	for (i = 0; i < s3minfo.insnum; i++)
	{
		theInstrument[ i] = 0L;
	
		theS3MCopy = (Byte*) theS3M;
		theS3MCopy += s3minfo.parapins[i]*16L;
		
		BlockMove( theS3MCopy, &s3minfo.insdata[i], sizeof(s3minsform));
		
		s3minfo.insdata[i].insmemseg		= Tdecode16( &s3minfo.insdata[i].insmemseg);
		s3minfo.insdata[i].inslength		= Tdecode32( &s3minfo.insdata[i].inslength);
		
		if( s3minfo.insdata[i].insflags&1)
		{
			s3minfo.insdata[i].insloopbeg		= Tdecode32( &s3minfo.insdata[i].insloopbeg);
			s3minfo.insdata[i].insloopend		= Tdecode32( &s3minfo.insdata[i].insloopend);
		}
		else
		{
			s3minfo.insdata[i].insloopbeg		= 0;
			s3minfo.insdata[i].insloopend		= 0;
		}
		
		s3minfo.insdata[i].insloc2spd		= Tdecode16(  &s3minfo.insdata[i].insloc2spd);
		s3minfo.insdata[i].inshic2spd		= Tdecode16(  &s3minfo.insdata[i].inshic2spd);
		s3minfo.insdata[i].insgvspos		= Tdecode16(  &s3minfo.insdata[i].insgvspos);
		s3minfo.insdata[i].insint512		= Tdecode16(  &s3minfo.insdata[i].insint512);
		s3minfo.insdata[i].insintlastused	= Tdecode32( &s3minfo.insdata[i].insintlastused);
		
		if (s3minfo.insdata[i].instype == 1 && s3minfo.insdata[i].inspack == 0 && s3minfo.insdata[i].inssig == 'SCRS')
	        {
	        	theS3MCopy = (Byte*) theS3M;
	        	theS3MCopy += s3minfo.insdata[i].insmemseg*16L;
	        	
	        	theInstrument[ i] = (Ptr) theS3MCopy;
	        }
	        else
	        {
	        	theInstrument[ i] = 0L;
	        }
	}

	/******** Le S3M a t lu et analys ***********/
	/******** Copie des informations dans le MAD ***/
	
	theMAD->header = (MADSpec*) NewPtrClear( sizeof( MADSpec));	
	theMAD->header->MAD = 'MADH';
	for(i=0; i<32; i++) theMAD->header->name[i] = 0;
	for(i=0; i<28; i++) theMAD->header->name[i] = s3minfo.name[i];

	theMAD->header->numPat		= s3minfo.patnum;
	theMAD->header->numPointers	= s3minfo.ordernum;
	theMAD->header->speed		= s3minfo.initialspeed;
	theMAD->header->tempo		= s3minfo.initialtempo;
	
	for(i=0; i<128; i++) theMAD->header->oPointers[ i] = 0;
	for(i=0; i<s3minfo.ordernum; i++)
	{
		theMAD->header->oPointers[ i] = s3minfo.orders[i];
		
		if( theMAD->header->oPointers[ i] < 0 || theMAD->header->oPointers[ i] > s3minfo.patnum) theMAD->header->oPointers[ i] = 0;
	}

	/********************/
	/***** INSTRUMENTS *****/
	/********************/

	for(i  = 0 ; i < MAXINSTRU; i++)
	{
		for( x = 0; x < MAXSAMPLE; x++) theMAD->sample[ i][ x] = 0L;

		theMAD->header->fid[i].numSamples	= 0;
	}

	for(i=0; i<s3minfo.insnum; i++)
	{
		InstrData		*curIns = &theMAD->header->fid[ i];
		
		curIns->type	= 0;
		
		if( theInstrument[ i] != 0L)
		{
			sData	*curData;

			curIns->numSamples = 1;
			

			curData = theMAD->sample[ i][ 0] = (sData*) NewPtrClear( sizeof( sData));
			
			curData->size		= s3minfo.insdata[i].inslength;
			curData->loopBeg 	= s3minfo.insdata[i].insloopbeg;
			curData->loopSize 	= s3minfo.insdata[i].insloopend - s3minfo.insdata[i].insloopbeg;
			curData->vol		= s3minfo.insdata[i].insvol;
			curData->c2spd		= s3minfo.insdata[i].insloc2spd;
			curData->loopType	= 0;
			curData->amp		= 8;		if( s3minfo.insdata[i].insflags&4)	curData->amp		= 16;
			curData->panning	= 0;
			curData->relNote	= 0;
			for( x = 0; x < 28; x++) curData->name[ x] = s3minfo.insdata[i].insname[x];
			
			curData->data 		= NewPtr( curData->size);
			if( curData->data == 0L) DebugStr("\pInstruments: I NEED MEMORY !!! NOW !");
			
			if( curData->data != 0L)
			{
				BlockMove( theInstrument[i], curData->data, curData->size);
				
				if( s3minfo.ffv != 1)
				{
					for( temp = 0; temp < curData->size; temp++) *(curData->data + temp) -= 0x80;
				}
			}
		}
		else curIns->numSamples = 0;
	}
	/********************/
	
	/*********************/
	/*           Check MaxTrack         */
	/*********************/
	maxTrack = 0;
	for( i = 0; i < theMAD->header->numPat ; i++)
	{
		Row = 0;
		
		if( s3minfo.parappat[ i] > 0)
		{
			theS3MCopy = (Byte*) theS3M;
			theS3MCopy += ( (long) s3minfo.parappat[i] )*16L;
			theS3MCopy++;
			
			while( Row < 64)
			{
				tempChar = *theS3MCopy;
				theS3MCopy++;
	
				if( tempChar == 0) Row++;
				else
				{	// Channel
					channel = tempChar;
					channel &= 31;
					
					if( channel > maxTrack) maxTrack = channel;
					if( (tempChar & 32) != 0) theS3MCopy += 2L;
					if( (tempChar & 64) != 0) theS3MCopy += 1L;
					if( (tempChar & 128) != 0) theS3MCopy += 2L;
				}
			}
		}
	}
	maxTrack ++;
	
	
	/** Pair **/
	maxTrack++;
	maxTrack /= 2;
	maxTrack *= 2;
	/********************/
	/***** TEMPORAIRE ******/
	/********************/
	
	theMAD->header->numChn = maxTrack;
	
	starting = 0;

	for( i = 0; i < MAXPATTERN; i++) theMAD->partition[ i] = 0L;
	for( i = 0; i < theMAD->header->numPat ; i++)
	{
		theMAD->partition[ i] = (PatData*) NewPtrClear( sizeof( PatData) + theMAD->header->numChn * 64L * sizeof( Cmd));
		if( theMAD->partition[ i] == 0L) return MADNeedMemory;
		
		theMAD->partition[ i]->header.size = 64L;
		theMAD->partition[ i]->header.compMode = 'NONE';
		
		for( x = 0; x < 20; x++) theMAD->partition[ i]->header.name[ x] = 0;
	
		theMAD->partition[ i]->header.patBytes = 0L;
		theMAD->partition[ i]->header.unused2 = 0L;
	
		MaxPtr = (Ptr) theMAD->partition[ i];
		MaxPtr += sizeof( PatData) + theMAD->header->numChn * 64L * sizeof( Cmd);
		
		for( Row = 0; Row < 64; Row++)
		{
			for(z = 0; z < theMAD->header->numChn; z++)
			{
				aCmd = GetMADCommand( Row, z, theMAD->partition[ i]);
				
				aCmd->note		= 0xFF;
				aCmd->ins			= 0;
				aCmd->cmd		= 0;
				aCmd->arg		= 0;
				aCmd->vol		= 0xFF;
			}
		}
		
		if( s3minfo.parappat[ i] > 0)
		{
			theS3MCopy = (Byte*) theS3M;
			theS3MCopy += ( (long) s3minfo.parappat[i] )*16L;
			theS3MCopy++;
			
			Row = 0;
			while( Row < 64)
			{
				/*
					BYTE:flag, 	0		=	end of row
								&31		=	channel
								&32		=	follows;  BYTE:note, BYTE:instrument
								&64		=	follows;  BYTE:volume
								&128	=	follows; BYTE:command, BYTE:info
				*/
					
					tempChar = *theS3MCopy;
					theS3MCopy++;
	
					if( tempChar == 0) Row++;
					else
					{
						// Channel
						
						channel = tempChar;
						channel &= 31;
						if( channel >= 0 && channel < theMAD->header->numChn) z = channel;
						else DebugStr("\pOut - Channel");
						
						aCmd = GetMADCommand( Row, z, theMAD->partition[ i]);
						
						// PERIOD
						
						if( (tempChar & 32) != 0)
						{
							aCmd->note = theS3MCopy[ 0];
							
							Octave = (aCmd->note & 0xF0) >> 4;
							Note = (aCmd->note & 0x0F);
								
							aCmd->note = Octave*12 + Note;
							if( aCmd->note < 0 || aCmd->note >= NUMBER_NOTES) aCmd->note = 0xFF;
							
							aCmd->ins = theS3MCopy[ 1];
						//	if( theMAD->instrument[ theS3MCopy[ 1] - 1] != 0) aCmd->InstrumentNo = theS3MCopy[ 1];
							
							theS3MCopy += 2L;
						}
						
						// VOLUME
						
						if( (tempChar & 64) != 0)
						{
							aCmd->vol = theS3MCopy[ 0];
							if( aCmd->vol > 64) aCmd->vol = 64;
							aCmd->vol += 0x10;
							
							theS3MCopy += 1L;
						}
						else aCmd->vol = 255;
						
						// PARAMETER
						
						if( (tempChar & 128) != 0)
						{
							if( theS3MCopy[ 0] != 255)
							{
								ConvertS3MEffect( theS3MCopy[ 0], theS3MCopy[ 1], &aCmd->cmd, &aCmd->arg);
							}
							theS3MCopy += 2L;
						}
					}
			}
		}
	}
	
	DisposPtr( (Ptr) s3minfo.orders);		DisposPtr( (Ptr) s3minfo.parapins);
	DisposPtr( (Ptr) s3minfo.parappat);		DisposPtr( (Ptr) s3minfo.insdata);

	return noErr;
}

OSErr ExtractS3MInfo( PPInfoRec *info, Ptr AlienFile)
{
	s3mform		*myS3M = ( s3mform*) AlienFile;
	long			PatternSize;
	short		i;
	short		maxInstru;
	short		tracksNo;
	s3mform		s3minfo;
	/********************************/

	/**** Header principal *****/
	BlockMove( AlienFile, &s3minfo, 96);
	
	/*** Signature ***/
	
	info->signature = 'S3M ';
	
	/*** Internal name ***/
	
	myS3M->name[ 27] = '\0';
	pStrcpy( info->internalFileName, CtoPstr( myS3M->name));

	/*** Total Patterns ***/
	
	info->totalPatterns = 0;
	
	/*** Partition Length ***/
	
	info->partitionLength = 0;
	
	/*** Total Instruments ***/
	
	info->totalInstruments = Tdecode16(  &s3minfo.insnum);
	
	pStrcpy( info->formatDescription, "\pS3M Plug");

	return noErr;
}

OSErr TestS3MFile( Ptr AlienFile)
{
s3mform	*myS3M = ( s3mform*) AlienFile;

if(	myS3M->s3msig[ 0] == 'S' &&
	myS3M->s3msig[ 1] == 'C' &&
	myS3M->s3msig[ 2] == 'R' &&
	myS3M->s3msig[ 3] == 'M') return   noErr;
	else return  MADFileNotSupportedByThisPlug;
}

OSErr main( OSType order, FSSpec *AlienFileFSSpec, MADPartition *MadFile, PPInfoRec *info)
{
	OSErr	myErr;
	Ptr		AlienFile;
	short	vRefNum, iFileRefI;
	long	dirID, sndSize;
	
#ifndef powerc
	long	oldA4 = SetCurrentA4(); 			//this call is necessary for strings in 68k code resources
#endif

	HGetVol( 0L, &vRefNum, &dirID);
	HSetVol( 0L, AlienFileFSSpec->vRefNum, AlienFileFSSpec->parID);

	myErr = noErr;

	switch( order)
	{
		case 'IMPL':
			myErr = FSOpen( AlienFileFSSpec->name, 0, &iFileRefI);
			if( myErr == noErr)
			{
				GetEOF( iFileRefI, &sndSize);
			
				// ** MEMORY Test Start
				AlienFile = NewPtr( sndSize * 2L);
				if( AlienFile == 0L) myErr = MADNeedMemory;
				// ** MEMORY Test End
				
				else
				{
					DisposPtr( AlienFile);
					
					AlienFile = NewPtr( sndSize);
					myErr = FSRead( iFileRefI, &sndSize, AlienFile);
					if( myErr == noErr)
					{
						myErr = TestS3MFile( AlienFile);
						if( myErr == noErr)
						{
							myErr = ConvertS3M2Mad( AlienFile,  GetPtrSize( AlienFile), MadFile);
						}
					}
					DisposPtr( AlienFile);	AlienFile = 0L;
				}
				FSClose( iFileRefI);
			}
		break;
		
		case 'TEST':
			myErr = FSOpen( AlienFileFSSpec->name, 0, &iFileRefI);
			if( myErr == noErr)
			{
				sndSize = 1024L;
				
				AlienFile = NewPtr( sndSize);
				if( AlienFile == 0L) myErr = MADNeedMemory;
				else
				{
					myErr = FSRead( iFileRefI, &sndSize, AlienFile);
					myErr = TestS3MFile( AlienFile);
					
					DisposPtr( AlienFile);	AlienFile = 0L;
				}
				FSClose( iFileRefI);
			}
		break;

		case 'INFO':
			myErr = FSOpen( AlienFileFSSpec->name, 0, &iFileRefI);
			if( myErr == noErr)
			{
				GetEOF( iFileRefI, &info->fileSize);
			
				sndSize = 5000L;	// Read only 5000 first bytes for optimisation
				
				AlienFile = NewPtr( sndSize);
				if( AlienFile == 0L) myErr = MADNeedMemory;
				else
				{
					myErr = FSRead( iFileRefI, &sndSize, AlienFile);
					if( myErr == noErr)
					{
						myErr = ExtractS3MInfo( info, AlienFile);
					}
					DisposPtr( AlienFile);	AlienFile = 0L;
				}
				FSClose( iFileRefI);
			}
		break;
		
		default:
			myErr = MADOrderNotImplemented;
		break;
	}

	HSetVol( 0L, vRefNum, dirID);

	#ifndef powerc
		SetA4( oldA4);
	#endif
	return myErr;
}