#include <SATcpp.h>
#include <pixmanip.h>
#include <GrafMap.h>
#include <cFace.h>
#include <cPort.h>

// ************
// Initializers
// ************

void cFace::Init(const cFace &face)
{
	Init(face.iconMask.bounds);
	*this = face;
}

void cFace::Init(short colorPict, short bwPict, short maskPictnum)
{
	InitProc();

	cPort oldPort;
	oldPort.Get();
	
	PicHandle thePict = GetPicture((gSAT.initDepth > 1) ? colorPict : bwPict);
	PicHandle maskPict = GetPicture(maskPictnum);
	
	Rect bounds = (*thePict)->picFrame;
	bounds.right -= bounds.left;
	bounds.bottom -= bounds.top;
	*((long *) &topLeft(bounds)) = 0;

	Init(bounds);
	
	PrepDrawFace();
	DrawPicture(thePict, &bounds);
	PrepDrawMask();
	DrawPicture(maskPict, &bounds);
	
	Changed();
	
	ReleaseResource((Handle) thePict);
	ReleaseResource((Handle) maskPict);
	
	oldPort.Set();
}

cFace *cFace::Copy(void)
{
	void *source = GetBAddr();
	cFace *temp = new cFace(iconMask.bounds);
	BlockMove((Ptr) source, (Ptr) temp->GetBAddr(), GetPtrSize((Ptr) source));
	BlockMove(iconMask.baseAddr, temp->iconMask.baseAddr, GetPtrSize(iconMask.baseAddr));
	return(temp);
}

//
// -- Access Face stats
//

void *cFace::GetBAddr(void)
{
	cPort oldPort;
	oldPort.Get();
	
	PrepDrawFace();
	void *ptr = GetBaseAddr(qd.thePort);

	oldPort.Set();
	return(ptr);
}

short cFace::GetRBytes(void) const
{
	cPort oldPort;
	oldPort.Get();
	
	PrepDrawFace();
	short r = GetRowBytes(qd.thePort);

	oldPort.Set();
	return(r);
}

short cFace::GetDepth(void) const
{
	return(GetRBytes() / (iconMask.rowBytes));
}

//
// -- Manipulate Face
//

void cFace::FlipRows(void)
{
	short height = GetHeight();
	flip_rows((Ptr) GetBAddr(), GetRBytes(), height);
	flip_rows(iconMask.baseAddr, iconMask.rowBytes, height);
	Changed();
}

void cFace::RotRight()
{
	short height = GetHeight();
	short depth = GetDepth();

	rotate_right((Ptr) GetBAddr(), GetRBytes(), height, depth);
	rotate_right(iconMask.baseAddr, iconMask.rowBytes, height, 1);
	Changed();
}

void cFace::FlipColumns()
{
	short height = GetHeight();
	short depth = GetDepth();
	if (!depth)
		depth = 1;

	flip_columns((Ptr) GetBAddr(), GetRBytes(), height, depth);
	flip_columns(iconMask.baseAddr, iconMask.rowBytes, height, 1);
	Changed();
}

void cFace::RotLeft()
{
	register short height = GetHeight();
	register short depth = GetDepth();

	rotate_left((Ptr) GetBAddr(), GetRBytes(), height, depth);
	rotate_left(iconMask.baseAddr, iconMask.rowBytes, height, 1);
	Changed();
}

void cFace::Clear()
{
	cPort oldPort;
	oldPort.Get();
	
	PrepDrawFace();
	EraseRect(&GetBounds());
	PrepDrawMask();
	EraseRect(&GetBounds());
	
	oldPort.Set();
}
