#pragma once

#include <SATcpp.h>
#include <cPort.h>
#include <stdlib.h>

class cFace;

class cFace : private Face {
public:
	typedef void (*RotFunc)();
	
	cFace() {																// null face
		rowBytes = 0;
	}
	cFace(const Rect &faceBounds) {											// new face
		Init(faceBounds);
	}
	cFace(short iconNum) {													// face from cicn
		Init(iconNum);
	}
	cFace(short colorPict, short bwPict, short maskPictnum) {				// face from pict
		Init(colorPict, bwPict, maskPictnum);
	}
	cFace(const cFace &face) {												// face from face
		Init(face);
	}
	cFace(short iconNum, RotFunc func) {									// Allow Rotating
		Init(iconNum);
		func();
	}
	cFace(short colorPict, short bwPict, short maskPictnum, RotFunc func) {
		Init(colorPict, bwPict, maskPictnum);
		func();
	}
	cFace(const cFace &face, RotFunc func) {
		Init(face);
		func();
	}
	const cFace &operator = (const cFace &him) {
		SATCopyFace(this, him.FaceStruct());
		return him;
	}
	~cFace() {
		Reset();
	}
	void Reset() {
		if (rowBytes) {
			SATDisposeFacePP(FaceStruct());
			rowBytes = 0;
		}
	}
	void Set(short iconNum) {
		Reset();
		Init(iconNum);
	}
	void Set(short colorPict, short bwPict, short maskPictnum) {
		Reset();
		Init(colorPict, bwPict, maskPictnum);
	}
	void Set(short iconNum, RotFunc func) {
		Reset();
		Init(iconNum);
		func();
	}
	void Set(short colorPict, short bwPict, short maskPictnum, RotFunc func) {
		Reset();
		Init(colorPict, bwPict, maskPictnum);
		func();
	}
	void Set(const Rect &faceBounds) {
		Reset();
		Init(faceBounds);
	}
	void FlipRows();
	void FlipColumns();
	void RotLeft();
	void RotRight();
	void Clear();
	void Changed() {
		SATChangedFace(this);
	}
	void PrepDrawFace(void) const {
		SATSetPortFace((Face *) (const Face *) FaceStruct());
	}
	void PrepDrawMask() const {
		SATSetPortMask((Face *) (const Face *) FaceStruct());
	}
	short GetHeight() const {
		return iconMask.bounds.bottom - iconMask.bounds.top;
	}
	short GetWidth() const {
		return iconMask.bounds.right - iconMask.bounds.left;
	}
	Point GetHW() const {
		Point p;
		p.v = GetHeight();
		p.h = GetWidth();
		return p;
	}
	short GetDepth() const;
	void *GetBAddr();
	short GetRBytes() const;
	
	void Plot(cPort *port, Point where, Boolean fast) {
		SATPlotFace(FaceStruct(), port, where, fast);
	}
	void PlotToScreen(Point where, Boolean fast) {
		SATPlotFaceToScreen(FaceStruct(), where, fast);
	}
protected:
	cFace *Copy();
	const Rect &GetBounds() const {
		return iconMask.bounds;
	}
private:
	Face *FaceStruct() {
		return this;
	}
	const Face *FaceStruct() const {
		return this;
	}
	void Init(const Rect &faceBounds) {
		SATNewFaceProc((Rect *) &faceBounds, (Ptr) FaceStruct());
	}
	void Init(short iconNum) {
		SATGetFaceProc(iconNum, (Ptr) FaceStruct());
	}
	void Init(short colorPict, short bwPict, short maskPictnum);
	void Init(const cFace &face);
};
