#include <sSprite.h>

static Boolean destructManually = TRUE;

static pascal void HandleSprite(SpritePtr me)
{
	ExtractSprite(me)->Update();
}

static pascal void HitSprite(SpritePtr me, SpritePtr him)
{
	ExtractSprite(me)->Hit(ExtractSprite(him));
}

static pascal void DestructSprite(SpritePtr me)
{
	destructManually = FALSE;
	if (GetApplLimit() > (Ptr) me)					//	Check to see if allocated on heap
		delete ((sSprite *) me->spriteBegin);
	else
		((sSprite *) me->spriteBegin)->~sSprite();
	destructManually = TRUE;
}

static pascal void SetupSprite(SpritePtr me) {
	me->task = HandleSprite;
	me->hitTask = HitSprite;
	me->destructTask = DestructSprite;
}

sSprite::sSprite(const sSprite &afterthis, short newkind, Point p)
{
	Init(&afterthis, newkind, p);
}

sSprite::sSprite(const sSprite &afterthis, short newkind)
{
	Init(&afterthis, newkind, afterthis.GetPos());
}

sSprite::sSprite(short newkind, Point p)
{
	Init(0, newkind, p);
}

void sSprite::Init(const sSprite *afterthis, short newkind, Point p)
{
	ClearMem(SpriteStruct(), sizeof(Sprite));
	SATNewSpritePP(afterthis ? (Sprite *) (afterthis->SpriteStruct()) : 0, (Ptr) SpriteStruct(), newkind, p.h, p.v, SetupSprite);
	SpriteStruct()->spriteBegin = (Ptr) this;
}

void *sSprite::CheckSignat(Signat s) const
{
	return 0;
}

void sSprite::Remove()
{
	SpriteStruct()->task = 0;
}

void sSprite::Dispose()
{
}

void sSprite::Hit(sSprite *him)
{
}

void sSprite::Update()
{
}

sSprite::~sSprite()
{
	if (SpriteStruct()->hitTask == 0)			// Already been disposed
		return;
	SpriteStruct()->hitTask = 0;
	Dispose();
	if (SpriteStruct()->task)					// If not removed by SAT...
		Kill();
}

void sSprite::Kill(void)
{
	Rect r;
	topLeft(r) = botRight(r) = GetPos();
	r.bottom += GetHeight();
	r.right += GetWidth();
	SATBackChanged(&r);
	SATKillSprite(SpriteStruct());
}

sSprite *sSprite::GetNext()
{
	register SpritePtr next = SpriteStruct();
	do {
		next = next->next;
		if (!next)
			return 0;
	}
	while(next->task == 0);
	return ExtractSprite(next);
}
