#pragma once

#include <SATcpp.h>
#include <stdlib.h>
#include <cFace.h>
#include <PointUtils.h>

const short inert = 0;

#define MembExtract(sprite, class)	((class*) (sprite).CheckSignat(signat_##class))
#define CompSignat(sig, class)		((sig) == signat_##class)
#define MatchSignat(sig, class)	if (CompSignat(sig, class)) return (void *) this;

class sSprite;

inline sSprite *ExtractSprite(Sprite *s)
{
	return ((sSprite *) s->spriteBegin);
}

#define gSpriteRoot (ExtractSprite(gSAT.sRoot))

class sSprite {
public:
	// Constructors/Destructors/Operators
	sSprite(const sSprite &afterthis, short newkind, Point p);
	sSprite(const sSprite &afterthis, short newkind);
	sSprite(short newkind, Point p);
	
	~sSprite();
	const sSprite &operator = (const sSprite &him) {
		SATCopySprite(SpriteStruct(), him.SpriteStruct());
		return him;
	}
	
	// Virtuals
	virtual void Update();
	virtual void Hit(sSprite *him);
	virtual void Remove();
	virtual void *CheckSignat(Signat s) const;
	virtual void Dispose();
	
	// Accessors
	void SetKind(short k) {
		SpriteStruct()->kind = k;
	}
	short GetKind() const {
		return SpriteStruct()->kind;
	}
	void SetLayer(short l) {
		SpriteStruct()->layer = l;
	}
	short GetLayer() const {
		return SpriteStruct()->layer;
	}
	short GetHeight() const {
		return GetFace()->GetHeight();
	}
	short GetWidth() const {
		return GetFace()->GetWidth();
	}
	Point GetHW() const {
		return GetFace()->GetHW();
	}
	void SetHotRect(const Rect *r) {
		SpriteStruct()->hotRect.top = r->top;
		SpriteStruct()->hotRect.left = r->left;
		SpriteStruct()->hotRect.bottom = r->bottom;
		SpriteStruct()->hotRect.right = r->right;
//		SpriteStruct()->hotRect = r;
	}
	void SetPos(Point p) {
		SpriteStruct()->position = p;
	}
	void SetHPos(short h) {
		SpriteStruct()->position.h = h;
	}
	void SetVPos(short v) {
		SpriteStruct()->position.v = v;
	}
	Point GetPos() const {
		return SpriteStruct()->position;
	}
	short GetHPos() const {
		return SpriteStruct()->position.h;
	}
	short GetVPos() const {
		return SpriteStruct()->position.v;
	}
	const Rect &GetHotRect() const {
		return SpriteStruct()->hotRect;
	}
	Rect GetPlaceRect() const {
		return GetHotRect() + GetPos();
	}
	Boolean Overlaps(const Rect &rect) {
		return rect && GetPlaceRect();
	}
	void SetFace(cFace &f) {
		SpriteStruct()->face = &f;
	}
	cFace *GetFace() const {
		return (cFace *) SpriteStruct()->face;
	}
	Boolean IsAlive() {
		return SpriteStruct()->hitTask != 0;
	}
	void DumpFace(cFace *theFace, cPort *port, Boolean fast) {
		theFace->Plot(port, GetPos(), fast);
	}
	void DumpSpriteFace(cPort *port, Boolean fast) {
		DumpFace(GetFace(), port, fast);
	}
	void DumpFaceToScreen(cFace *theFace, Boolean fast) {
		theFace->PlotToScreen(GetPos(), fast);
	}
	void DumpSpriteFaceToScreen(Boolean fast) {
		DumpFaceToScreen(GetFace(), fast);
	}
	void Bump(Point vec) {
		SetPos(GetPos() + vec);
	}
	void SetMousePos() {
		Point p;
		GetMouse(&p);
		SetPos(p);
	}
	Boolean HoldTop(short top) {
		if (GetVPos() >=top)
			return FALSE;
		SetVPos(top);
		return TRUE;
	}
	Boolean HoldBottom(short bottom) {
		if (GetVPos() <= bottom)
			return FALSE;
		SetVPos(bottom);
		return TRUE;
	}
	Boolean HoldLeft(short left) {
		if (GetHPos() >= left)
			return FALSE;
		SetHPos(left);
		return TRUE;
	}
	Boolean HoldRight(short right) {
		if (GetHPos() <= right)
			return FALSE;
		SetHPos(right);
		return TRUE;
	}
	Boolean TouchingRect(const Rect &rect, Rect *sect = 0) {
		Rect dummy;
		return SectRect(&rect, &GetPlaceRect(), sect ? sect : &dummy);
	}
	Boolean operator && (const Rect &rect) {
		return TouchingRect(rect);
	}
	sSprite *GetNext();
private:
	void Kill();
	void Init(const sSprite *afterthis, short newkind, Point p);
	Sprite *SpriteStruct() {
		return &theSprite;
	}
	const Sprite *SpriteStruct() const {
		return (const Sprite *) &theSprite;
	}
	Sprite theSprite;
};
