// Point struct utilities

#pragma once

inline Point POINT(short h, short v)	// Inline to generate point from h/v coords
{
	Point p;
	p.h = h;
	p.v = v;
	return p;
}

inline Rect RECT(short left, short top, short right, short bottom)
{
	Rect r;
	r.top = top;
	r.left = left;
	r.bottom = bottom;
	r.right = right;
	return r;
}

// Point structures are used for vectors as well, so lets make VECTOR a synonym
// for POINT.  After all, "Bump(VECTOR(1,0))" looks nicer than "Bump(POINT(1,0))"

#define VECTOR(h,v) POINT(h,v)

inline long PTLONG(Point p)				// Inline to use Point as long
{
	return *((long *) &p);
}

inline Point LONGPT(long l)				// Inline to use long as Point
{
	return *((Point *) &l);
}

// Operators on Points -

inline Point operator + (const Point &pt1, const Point &pt2)
{
	Point p;
	p.h = pt1.h + pt2.h;
	p.v = pt1.v + pt2.v;
	return p;
}

inline Point operator += (Point pt1, const Point pt2)
{
	pt1.h += pt2.h;
	pt1.v += pt2.v;
	return pt1;
}

inline Point operator - (const Point pt1, const Point pt2)
{
	Point p;
	p.h = pt1.h - pt2.h;
	p.v = pt1.v - pt2.v;
	return p;
}

inline Point operator -= (Point pt1, const Point pt2)
{
	pt1.h -= pt2.h;
	pt1.v -= pt2.v;
	return pt1;
}

inline Point operator * (const Point pt, const short n)
{
	Point p;
	p.h = pt.h * n;
	p.v = pt.v * n;
	return p;
}

inline Point operator *= (Point pt, const short n)
{
	pt.h *= n;
	pt.v *= n;
	return pt;
}

inline Point operator / (const Point pt, const short n)
{
	Point p;
	p.h = pt.h / n;
	p.v = pt.v / n;
	return p;
}

inline Point operator /= (Point pt, const short n)
{
	pt.h /= n;
	pt.v /= n;
	return pt;
}

inline int operator == (const Point pt1, const Point pt2)
{
	return PTLONG(pt1) == PTLONG(pt2);
}

inline int operator != (const Point pt1, const Point pt2)
{
	return PTLONG(pt1) != PTLONG(pt2);
}

inline Rect operator + (const Rect &r1, const Rect &r2)
{
	Rect r;
	r.top = r1.top + r2.top;
	r.left = r1.left + r2.left;
	r.bottom = r1.bottom + r2.bottom;
	r.right = r1.right + r2.right;
	return r;
}

inline Rect operator + (const Rect &r1, const Point p)
{
	Rect r;
	r.top = r1.top + p.v;
	r.left = r1.left + p.h;
	r.bottom = r1.bottom + p.v;
	r.right = r1.right + p.h;
	return r;
}

inline Rect operator += (Rect &r1, const Rect &r2)
{
	r1.top += r2.top;
	r1.left += r2.left;
	r1.bottom += r2.bottom;
	r1.right += r2.right;
	return r1;
}

inline Rect operator + (Rect &r1, const Point p)
{
	r1.top += p.v;
	r1.left += p.h;
	r1.bottom += p.v;
	r1.right += p.h;
	return r1;
}

// Note that the first parameter MUST be the enclosing rectangle!

inline int operator && (const Rect &r1, const Rect &r2)
{
	return (r1.top <= r2.top) && (r1.bottom >= r2.bottom) &&
			(r1.left <= r2.left) && (r1.right >= r2.right);
}

inline int operator || (const Rect &r1, const Rect &r2)
{
	Rect dummy;
	return SectRect(&r1, &r2, &dummy);
}
