#include <string.h>
#include <swap.h>

void flip_rows(char *map, int row_bytes, int height);
int getpixpos(char *&map, int row_bytes, int depth, int x, int y);
char getmappix(char *map, int row_bytes, int depth, int x, int y);
void setmappix(char *map, int row_bytes, int depth, int x, int y, char pix);
void adj_right(int &x, int &y, int height, int width);
void adj_left(int &x, int &y, int height, int width);
void adj_flip_columns(int &x, int &y, int height, int width);
void manip(char *map, int row_bytes, int height, int depth, void (*adjust)(int&, int&, int, int));

void flip_rows(char *map, int row_bytes, int height)
{
	int i,j, range = height /2;

	for (i = 0; i<range; i++)
		for (j = 0; j<row_bytes; j++)
			swap(map[row_bytes * i + j], map[(height - i -1) * row_bytes + j]);
}

int getpixpos(char *&map, int row_bytes, int depth, int x, int y)
{
	int pixbyte = 8 / depth;
	map += (row_bytes * y + x / pixbyte);
	return(8 - ((x % pixbyte) +1) * depth);
}

char getmappix(char *map, int row_bytes, int depth, int x, int y)
{
	int pos = getpixpos(map, row_bytes, depth, x, y);
	char c = *map;
	c >>= pos;
	c &= ((1 << depth) -1);
	return(c);
}

void setmappix(char *map, int row_bytes, int depth, int x, int y, char pix)
{
	if (!pix)
		return;
	int pos = getpixpos(map, row_bytes, depth, x, y);

	pix <<= pos;
	*map |= pix;
}

void adj_right(int &x, int &y, int height, int width)
{
	int xi = x, yi = y;
	y = xi;
	x = height - yi - 1;
}

void adj_left(int &x, int &y, int height, int width)
{
	int xi = x, yi = y;
	x = yi;
	y = width - xi - 1;
}

void adj_flip_columns(int &x, int &y, int height, int width)
{
	x = width - x - 1;
}

void manip(char *map, int row_bytes, int height, int depth, void (*adjust)(int&, int&, int, int))
{
	size_t size = height * row_bytes;
	char *work = new char[size];
	int x, y, width = (row_bytes * 8) /depth;
	
	for (x = 0; x < size; x++)
		work[x] = 0;
	for (x = 0; x < width; x++)
		for (y = 0; y < height; y++) {
			int xi = x, yi = y;
			adjust(xi, yi, height, width);
			setmappix(work, row_bytes, depth, xi, yi, getmappix(map, row_bytes, depth, x, y));
		}
	memcpy(map, work, size);
	delete work;
}
