{****************************************************}
{}
{		CSATDirector.p																																																}
{}
{		Methods for the director class by which the THINK Class Library can be	used 		}
{		with the Sprite Animation Toolkit. 																																	}
{}
{****************************************************}


unit CSATDirector;

interface

	uses
		TCL, SATTCLIntf;

implementation


{****************************************************}
{}
{		ISATDirector																																																	}
{}
{		Initializes the SAT director class.																																		}
{}
{****************************************************}

	procedure CSATDirector.ISATDirector (aSupervisor: CApplication);

	begin { ISATDirector }
		IDirector(aSupervisor);

		itsSATPane := nil;
	end; { ISATDirector }


{****************************************************}
{}
{		Free																																																								}
{}
{		Free the SAT director class, and close down any sounds. This is a good place		}
{		to delete any sprites and sounds which you have created.																	}
{}
{****************************************************}

	procedure CSATDirector.Free;

	begin { Free }
		SATSoundShutup;

		itsSATPane := nil;

		inherited Free;
	end; { Free }


{****************************************************}
{}
{		BuildWindow																																																		}
{}
{		In this method, you should build the window as you want it, and set									}
{		itsSATPane to point to the single instance of the CSATPane class.										}
{}
{****************************************************}

	procedure CSATDirector.BuildWindow;

	begin { BuildWindow }
	end; { BuildWindow }


{****************************************************}
{}
{		SetUpSprites																																																		}
{}
{		In this method, you should build the sprites as you want them, using the SAT		}
{		commands.																																																			}
{}
{****************************************************}

	procedure CSATDirector.SetUpSprites;

	begin { SetUpSprites }
	end; { SetUpSprites }


{****************************************************}
{}
{		SetUpSounds																																																		}
{}
{		In this method, you should build the sounds as you want them, using the SAT		}
{		commands.																																																			}
{}
{****************************************************}

	procedure CSATDirector.SetUpSounds;

	begin { SetUpSounds }
	end; { SetUpSounds }


{****************************************************}
{}
{		PrepareForPlaying																																													}
{}
{		In this method, do things which should be done before the game loop begins.			}
{		The default method prepares the SAT pane for drawing.																		}
{}
{****************************************************}

	procedure CSATDirector.PrepareForPlaying;

	begin { PrepareForPlaying }
		if itsSATPane <> nil then begin
				itsSATPane.PrepareForSATRun;
			end; { if }
	end; { PrepareForPlaying }


{****************************************************}
{}
{		DoPlay																																																						}
{}
{		This method should implement the game loop.																										}
{}
{****************************************************}

	procedure CSATDirector.DoPlay;

	begin { DoPlay }
	end; { DoPlay }


{****************************************************}
{}
{		FinishPlaying																																																		}
{}
{		In this method, do things which should be done after the game loop ends.						}
{}
{****************************************************}

	procedure CSATDirector.FinishPlaying;

	begin { FinishPlaying }
	end; { FinishPlaying }


end. { CSATDirector }