/*
	WASTE Demo Project:
	Menu Handling
			
	Copyright  1993-1995 Marco Piovanelli
	All Rights Reserved

	C port by John C. Daub
*/

#ifndef __ALIASES__
#include <Aliases.h>
#endif
#ifndef __DEVICES__
#include <Devices.h>
#endif
#ifndef __LOWMEM__
#include <LowMem.h>
#endif
#ifndef __STANDARDFILE__
#include <StandardFile.h>
#endif
#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef __WEDEMOAPP__
#include "WEDemoIntf.h"
#endif

#include "WETabs.h"

// resource types

#define kTypeMenuColorTable		'mctb'

// static variables

static MenuCRsrcHandle		sColors;		// handle to the 'mctb' resource for the Color menu
static ModalFilterYDUPP		sMySFDialogFilter; // static variable for exclusive use by GetMySFDialogFilter

// constants used by DoClose()

enum {
	kButtonSave			= 1,
	kButtonCancel,
	kButtonDontSave
};



void	SetDefaultDirectory( const FSSpec *spec )
{
	LMSetCurDirStore( spec->parID );
	LMSetSFSaveDisk( -spec->vRefNum );
	
	return;
}

// in the original WASTE Demo App code, Marco just had MySFDialogFilter() call
// MyStandardDialogFilter().  However, due to how I extended the functionality
// of MyStandardDialogFilter(), we cannot do that, hence we need another filter for
// our SFDialogs.  (incidentally, this function is pretty much how Marco originally
// wrote MyStandardDialogFilter(), if you're interested)

pascal Boolean MySFDialogFilter( DialogRef dialog, EventRecord *event, short *item, void *yourData )
{
#pragma unused ( yourData )

	GrafPtr				savePort;
	ModalFilterUPP		stdFilter = NULL;
	Boolean				reply = false;
	OSErr				err;
	
	// set up the port
	
	GetPort( &savePort );
	SetGrafPortOfDialog( dialog );
	
	// 	intercept window events directed to widnows behind the dialog
	
	if ( ( event->what == updateEvt ) || ( event->what == activateEvt ) )
	{
		if ( (DialogRef)event->message != dialog )
			DoWindowEvent( event );
	}
	
	//	call the standard Dialog Manager filter procedure
	

	err = GetStdFilterProc( &stdFilter );
	if (err == noErr)
		reply = CallModalFilterProc( stdFilter, dialog, event, item );
	
	//	restore the port
	SetPort( savePort );
	
	return reply;

}


pascal ModalFilterYDUPP GetMySFDialogFilter( void )
{
	if ( sMySFDialogFilter == NULL )
		sMySFDialogFilter = NewModalFilterYDProc( MySFDialogFilter );
	
	return sMySFDialogFilter;
}


short	FindMenuItemText( MenuRef menu, ConstStr255Param stringToFind )
{
	short		item;
	Str255		itemString;
	
	for ( item = CountMItems( menu ); item >= 1; item-- )
	{
		GetMenuItemText( menu, item, itemString );
		if ( EqualString( itemString, stringToFind, false, false ) )
			break;
	}
	
	return item;
}

Boolean	EqualColor( const RGBColor *rgb1, const RGBColor *rgb2 )
{
	return ( (rgb1->red == rgb2->red) && (rgb1->green == rgb2->green) && (rgb1->blue == rgb2->blue) );
}


void	PrepareMenus( void )
{
	WindowRef		window;
	WEReference		we;
	MenuRef			menu;
	short			item;
	Str255			itemText;
	long			selStart, selEnd;
	WEActionKind	actionKind;
	short			mode;
	TextStyle		ts;
	Boolean			temp;
	
	// get a pointer to the frontmost window, if any
	
	window = FrontWindow();
	
	// get associated WE instance
	
	we = GetWindowWE(window);
	
	// *** FILE MENU ***
	
	menu = GetMenuHandle( kMenuFile );
	
	// first disable all items
	
	for ( item = CountMItems( menu); item >= 1; item-- )
		DisableItem( menu, item );
	
	// New, Open, and Quit are always enabled
	
	EnableItem( menu, kItemNew );
	EnableItem( menu, kItemOpen );
	EnableItem( menu, kItemQuit );
	
	// enable Close and Save As if there is an active window
	
	if ( window != NULL )
	{
		EnableItem( menu, kItemClose );
		EnableItem( menu, kItemSaveAs );
		
		// enable Save is the active window is dirty
		
		if ( WEGetModCount( we ) > 0 )
			EnableItem( menu, kItemSave );
	}
	
	// *** EDIT MENU ***
	
	menu = GetMenuHandle( kMenuEdit );
		
	// first, disable all items
	
	for ( item = CountMItems( menu ); item >= 1; item-- )
		DisableItem( menu, item );
	
	// by default, the Undo menu item should read "Can't Undo"
	
	GetIndString( itemText, kUndoStringsID, 1 );
	SetMenuItemText( menu, kItemUndo, itemText );
	
	if ( window != NULL )
	{
		// enable Paste if there's anything pasteable on the Clipboard
		
		if ( WECanPaste( we ) )
			EnableItem( menu, kItemPaste );
		
		// enable Undo if anything can be undone
		
		actionKind = WEGetUndoInfo( &temp, we );
		
		if ( actionKind != weAKNone )
		{
			EnableItem( menu, kItemUndo );
			
			// change the Undo menu item to "Undo/Redo" + name of action to undo/redo
			// the funky ?: thing is to replicate the Pascal ORD() function
			
			GetIndString( itemText, kUndoStringsID, 2 * actionKind + ( temp ? 1 : 0 ) );
			SetMenuItemText( menu, kItemUndo, itemText );
		}
		
		// enable Select All if there is anything to select
		
		if ( WEGetTextLength( we ) > 0 )
			EnableItem( menu, kItemSelectAll );
			
		// get the current selection range
		
		WEGetSelection( &selStart, &selEnd, we );
		if ( selStart != selEnd )
		{
			// enable Cut, Copy, and Clear if the selection range is not empty
			
			EnableItem( menu, kItemCut );
			EnableItem( menu, kItemCopy );
			EnableItem( menu, kItemClear );
		}
		
		// determine which style attributes are continuous over the current selection range
		// we'll need this information in order to check the Font/Size/Style/Color menus properly
		
		mode = weDoAll;  // query about all attributes
		temp = WEContinuousStyle( &mode, &ts, we );
		
	}
	else
		mode = 0;  // no window, so check no items
	
	// *** FONT MENU ***
	
	menu = GetMenuHandle( kMenuFont );
	
	// first, remove all check marks
	
	for ( item = CountMItems( menu ); item >= 1; item-- )
		CheckItem( menu, item, false );
	
	// if there is a continuous font all over the selection range, check the
	// corresponding menu item
	
	if ( (BAND( mode, weDoFont )) != 0 )
	{
		GetFontName( ts.tsFont, itemText );
		CheckItem( menu, FindMenuItemText( menu, itemText ), true );
	}
	
	// *** SIZE MENU ***
	
	menu = GetMenuHandle( kMenuSize );
	
	// first, remove all check marks
	
	for ( item = CountMItems( menu ); item >= 1; item-- )
		CheckItem( menu, item, false );
	
	// if tehre is a continuous font size all over the selection range
	// check the corresponding menu item
	
	if ( BAND( mode, weDoSize ) != 0 )
	{
		NumToString( ts.tsSize, itemText );
		CheckItem( menu, FindMenuItemText( menu, itemText ), true );
	}
	
	// *** STYLE MENU ***
	
	menu = GetMenuHandle( kMenuStyle );
	
	// first, remove all check marks
	for ( item = CountMItems( menu ); item >= 1; item-- )
		CheckItem( menu, item, false );
		
	// check the style menu items corresponding to style attributes
	
	if ( BAND( mode, weDoFace ) != 0 )
	{
		if ( ts.tsFace == 0 )
			CheckItem( menu, kItemPlainText, true );
		
		if ( ts.tsFace & bold )
			CheckItem( menu, kItemBold, true );
		
		if ( ts.tsFace & italic )
			CheckItem( menu, kItemItalic, true );
		
		if ( ts.tsFace & underline )
			CheckItem( menu, kItemUnderline, true );
		
		if ( ts.tsFace & outline )
			CheckItem( menu, kItemOutline, true );
		
		if ( ts.tsFace & shadow )
			CheckItem( menu, kItemShadow, true );
		
		if ( ts.tsFace & condense )
			CheckItem( menu, kItemCondensed, true );
		
		if ( ts.tsFace & extend )
			CheckItem( menu, kItemExtended, true );
		
	}
	
	// *** COLOR MENU ***
	
	menu = GetMenuHandle( kMenuColor );
	
	// first, remove all check marks
	
	for ( item = CountMItems( menu ); item >= 1; item-- )
		CheckItem( menu, item, false );
	
	// if there is a continuous color all over the selection range,
	// check the corresponding menu item (if any )

	if ( BAND( mode, weDoColor) != 0 )
	{
		for ( item = (*sColors)->numEntries - 1; item >= 0; item-- )
		{
			if ( EqualColor( &ts.tsColor, & (*sColors)->mcEntryRecs[item].mctRGB2 ) )
			{
				CheckItem( menu, (*sColors)->mcEntryRecs[item].mctItem, true );
			}	
	
		} // end for loop
	}
	
	// *** FEATURES MENU ***
	
	menu = GetMenuHandle( kMenuFeatures );
	
	// first remove all check marks (except the first item, which has a submenu!!)
	
	for ( item = CountMItems( menu ); item >= 2; item-- )
		CheckItem( menu, item, false );
	
	if ( window != NULL )
	{
		// mark each item according to the corresponding feature
		
		if ( WEIsTabHooks( we ) )
		{
			CheckItem( menu, kItemTabHooks, true );
			DisableItem( menu, kItemAlignment );
		}
		else
			EnableItem( menu, kItemAlignment );
		
		if ( WEFeatureFlag( weFAutoScroll, weBitTest, we ) )
			CheckItem( menu, kItemAutoScroll, true );
		
		if ( WEFeatureFlag( weFOutlineHilite, weBitTest, we ))
			CheckItem( menu, kItemOutlineHilite, true );
		
		if ( WEFeatureFlag( weFReadOnly, weBitTest, we ))
			CheckItem( menu, kItemReadOnly, true );
		
		if ( WEFeatureFlag( weFIntCutAndPaste, weBitTest, we ))
			CheckItem( menu, kItemIntCutAndPaste, true );
		
		if ( WEFeatureFlag( weFDragAndDrop, weBitTest, we ))
			CheckItem( menu, kItemDragAndDrop, true );
		
		if ( WEFeatureFlag( weFDrawOffscreen, weBitTest, we ) )
			CheckItem( menu, kItemOffscreenDrawing, true );
	}
	
	// *** ALIGNMENT MENU ***
	
	menu = GetMenuHandle( kMenuAlignment );
	
	// first, remove all check marks
	
	for ( item = CountMItems( menu ); item >= 1; item-- )
		CheckItem( menu, item, false );
	
	if ( window != NULL )
	{
		// find the Aligment menu item corresponding to mthe current alignment state
		
		switch ( WEGetAlignment( we ) )
		{
			case weFlushLeft:
				item = kItemAlignLeft;
			break;
			
			case weFlushRight:
				item = kItemAlignRight;
			break;
			
			case weFlushDefault:
				item = kItemAlignDefault;
			break;
			
			case weCenter:
				item = kItemCenter;
			break;
			
			case weJustify:
				item = kItemJustify;
			break;
			
			default:
				;  // nada
		}
		
		// check the menu item
		
		CheckItem( menu, item, true );
	}
	
	return;
}


void	DoDeskAcc( short menuItem )
{
	Str255		daName;
	short		daNumber;
	
	GetMenuItemText( GetMenuHandle( kMenuApple ), menuItem, daName );
	daNumber = OpenDeskAcc( daName );
	
	return;
}


OSErr	DoNew( void )
{
	// create a new window from scratch
	
	return CreateWindow( NULL );
}


OSErr	DoOpen( void )
{
	StandardFileReply		reply;
	SFTypeList				typeList;
	OSErr					err = noErr;
	Point					where = { -1, -1 };  // auto center dialog
	
	// set up a list of file types we can open for StandardGetFile
	
	typeList[0] = kTypeText;
	typeList[1] = kTypeTextReadOnly;
	
	// put up the standard open dialog box.
	// (we use CustomGetFile instead of StandardGetFile because we want to provide
	// our own dialog filter procedure that takes care of updating our windows)
	
	CustomGetFile( NULL, 1, typeList, &reply, 0, where, NULL, GetMySFDialogFilter(), NULL, NULL, NULL );
	
	// if the user ok'ed the dialog, create a new window from the specified file
	
	if ( reply.sfGood )
		err = CreateWindow( &reply.sfFile );
	else
		err = userCanceledErr;
	
	return err;
}


OSErr		SaveWindow( const FSSpec *pFileSpec, WindowRef window )
{
	DocumentHandle	hDocument;
	AliasHandle		aHandle;
	OSErr			err;
	
	hDocument = GetWindowDocument(window);
	
	ForgetHandle( & (*hDocument)->fileAlias );
	
	// svae the text
	
	err = WriteTextFile( pFileSpec, (*hDocument)->we );
	if ( err == noErr )
	{
		SetWTitle( window, pFileSpec->name );
		
		// replace the old window alias (if any) with a new one created from pFileSpec
		
		ForgetHandle( & (* hDocument)->fileAlias );
		err = NewAlias( NULL, pFileSpec, &aHandle );
		// if err, aHandle will be NULL, and it's not fatal, just will make subsequent saves annoying
		(* hDocument)->fileAlias = (Handle)aHandle;
	}
	
	return err;
}


OSErr	DoSaveAs( const FSSpec *suggestedTarget, WindowRef window )
{
	StringHandle		hPrompt;
	Str255				defaultName;
	StandardFileReply	reply;
	Point				where = { -1, -1 }; // autocenter's dialog
	OSErr				err;
	
	
	// get the prompt string for CustomPutFile from a 'STR ' resource and lock it
	
	hPrompt = GetString( kPromptStringID );
	HLockHi( (Handle)hPrompt );
	
	// if a suggested target file is provided, use its name as the default name
	
	if ( suggestedTarget != NULL )
	{
		PStringCopy( suggestedTarget->name, defaultName );
		SetDefaultDirectory( suggestedTarget );
	}
	else
		// otherwise use the window title as default name for CustomPutFile
		GetWTitle( window, defaultName );
	
	// put up the standard Save dialog box
	
	CustomPutFile( *hPrompt, defaultName, &reply, 0, where, NULL, GetMySFDialogFilter(), NULL, NULL, NULL );
	
	// unlock the string resource
	
	HUnlock( (Handle)hPrompt );
	
	// if the user ok'ed the dialog, save the window to the specified file
	
	if ( reply.sfGood )
		err = SaveWindow( &reply.sfFile, window );
	else
		err = userCanceledErr;
	
	return err;
}


OSErr	DoSave( WindowRef window )
{
	FSSpec		spec;
	FSSpecPtr	suggestedTarget = NULL;
	Boolean		promptForNewFile = true, aliasTargetWasChanged;
	OSErr		err;
	
	// resolve the alias associated with this window, if any
	
	if ( (* GetWindowDocument(window))->fileAlias != NULL )
	{
		if ( ( ResolveAlias( NULL, ((AliasHandle)(* GetWindowDocument(window))->fileAlias), &spec, &aliasTargetWasChanged ) == noErr ) )
		{
			if ( aliasTargetWasChanged )
				suggestedTarget = &spec;
			else
				promptForNewFile = false;
		}
	}
	
	// if no file has been previously associated with this window, or if the
	// alias resolution has failed, or if the alias target was changed
	// prompt the user for a new destination
	
	if ( promptForNewFile )
		err = DoSaveAs( suggestedTarget, window );
	else
		err = SaveWindow( &spec, window );
	
	return err;
}



OSErr		DoClose( ClosingOption closing, SavingOption saving, WindowRef window )
{
	Str255		s1, s2;
	short		alertResult;
	OSErr		err = noErr;
	
	// is this window dirty?
	
	if ( WEGetModCount( GetWindowWE(window) ) > 0 )
	{
		// do we have to ask the user whether to save changes?
		
		if ( saving == savingAsk )
		{
			// prepare the parametric strings to be used in the Save Changes alert box
			
			GetWTitle( window, s1 );
			// added this in over Marco's code - JCD
			
			if ( s1 == "\p" )  // if NULL
				PStringCopy( "\puntitled", s1 );
				
			GetIndString( s2, kClosingQuittingStringsID, 1 + (closing ? 1 : 0 ) );
			ParamText( s1, s2, "\p", "\p" );
			
			// put up the Save Changes? alert box
			
			SetCursor( &qd.arrow );
			
			alertResult = Alert( kAlertSaveChanges, GetMyStandardDialogFilter() );
			
			// exit if the user canceled the alert box
			
			if ( alertResult == kButtonCancel )
				return userCanceledErr;
			
			if ( alertResult == kButtonSave )
				saving = savingYes;
			else
				saving = savingNo;
		}
		
		if ( saving == savingYes )
		{
			err = DoSave( window );
			
			if ( err != noErr )
				return err;
			
		}
	}
	
	// destroy the window
	
	DestroyWindow( window );
	
	return err;
}



OSErr	DoQuit( SavingOption saving )
{
	WindowRef		window;
	OSErr			err = noErr;
	
	// close all windows
	
	do
	{
		window = FrontWindow();
		if ( window != NULL )
		{
			err = DoClose( closingApplication, saving, window );
			if ( err != noErr )
				return err;
		}
	} while ( window );
	
	// set a flag so we drop out of the event loop
	
	gExiting = true;
	
	return err;
}


void	DoAppleChoice( short menuItem )
{
	if ( menuItem == kItemAbout )
		DoAboutBox( kDialogAboutBox );
	else
		DoDeskAcc( menuItem );
	
	return;
}


void	DoFileChoice( short menuItem )
{
	WindowRef		window = FrontWindow();
	
	switch( menuItem )
	{
		case kItemNew:
			if ( DoNew() != noErr )
				;
		break;
		
		case kItemOpen:
			if ( DoOpen() != noErr )
				;
		break;
		
		case kItemClose:
			if ( DoClose( closingWindow, savingAsk, window ) != noErr )
				;
		break;
		
		case kItemSave:
			if ( DoSave( window ) != noErr )
				;
		break;
		
		case kItemSaveAs:
			if ( DoSaveAs( NULL, window ) != noErr )
				;
		break;
		
		case kItemQuit:
			if ( DoQuit( savingAsk ) != noErr )
				;
		break;
		
		default:
			;
		break;
	}
	
	return;
}


void	DoEditChoice( short menuItem )
{
	WindowRef		window;
	WEReference		we;
	
	// do nothing is no window is active
	
	window = FrontWindow();
	if ( window == NULL )
		return;
	
	we = GetWindowWE(window);
	
	switch ( menuItem )
	{
		case kItemUndo:
			if ( WEUndo(we) != noErr )
				;
		break;
		
		case kItemCut:
			if ( WECut( we) != noErr )
				;
		break;
		
		case  kItemCopy:
			if ( WECopy( we ) != noErr )
				;
		break;
		
		case kItemPaste:
			if ( WEPaste( we ) != noErr )
				;
		break;
		
		case kItemClear:
			if ( WEDelete( we ) != noErr )
				;
		break;
		
		case kItemSelectAll:
			WESetSelection( 0, LONG_MAX, we );
		break;
		
		default:
			;
		break;
	}
	
	return;
}


void	DoFontChoice( short menuItem )
{
	WindowRef		window;
	Str255			fontName;
	TextStyle		ts;
	
	window = FrontWindow();
	if ( window != NULL )
	{
		GetMenuItemText( GetMenuHandle( kMenuFont ), menuItem, fontName );
		GetFNum( fontName, &ts.tsFont );
		if ( WESetStyle( weDoFont, &ts, GetWindowWE(window) ) != noErr )
			;
	}
	
	return;
}


void	DoSizeChoice( short menuItem )
{
	WindowRef		window = FrontWindow();
	Str255			sizeString;
	long			longSize;
	short			mode;
	TextStyle		ts;
	
	if ( window != NULL )
	{
		if ( menuItem <= kItemLastSize )
		{
			GetMenuItemText( GetMenuHandle( kMenuSize ), menuItem, sizeString );
			StringToNum( sizeString, &longSize );
			mode = weDoSize;
			ts.tsSize = longSize;
		}
		else if ( menuItem == kItemSmaller )
		{
			mode = weDoAddSize;
			ts.tsSize = -1;
		}
		else if ( menuItem == kItemLarger )
		{
			mode = weDoAddSize;
			ts.tsSize = 1;
		}
		
		if ( WESetStyle( mode, &ts, GetWindowWE(window) ) != noErr )
			;
	}
	
	return;
}


void	DoStyleChoice( short menuItem )
{
	WindowRef		window = FrontWindow();
	TextStyle		ts;
	
	if ( window != NULL )
	{
		switch( menuItem )
		{
			case kItemPlainText:
				ts.tsFace = 0;
			break;
			
			case kItemBold:
				ts.tsFace = bold;
			break;
			
			case kItemItalic:
				ts.tsFace = italic;
			break;
			
			case kItemUnderline:
				ts.tsFace = underline;
			break;
			
			case kItemOutline:
				ts.tsFace = outline;
			break;
			
			case kItemShadow:
				ts.tsFace = shadow;
			break;
			
			case kItemCondensed:
				ts.tsFace = condense;
			break;
			
			case kItemExtended:
				ts.tsFace = extend;
			break;
			
			default:
				;
		}
		
		if ( WESetStyle( weDoFace + weDoToggleFace, &ts, GetWindowWE(window)) != noErr )
			;
	}
	
	return;
}


void	DoColorChoice( short menuItem )
{
	WindowRef			window = FrontWindow();
	short				i;
	TextStyle			ts;
	
	// do nothing if there is no front window
	
	if ( window == NULL )
		return;
	
	// find the color corresponding to the chosen menu item
	
	for ( i = (*sColors)->numEntries - 1; i >= 0; i-- )
	{
		if ( (*sColors)->mcEntryRecs[i].mctItem == menuItem )
		{
			ts.tsColor = (*sColors)->mcEntryRecs[i].mctRGB2;
			if ( WESetStyle( weDoColor, &ts, GetWindowWE(window) ) != noErr )
				;
		}
	}
	
	return;
}


void	DoAlignChoice( short menuItem )
{
	WindowRef		window = FrontWindow();
	SignedByte		alignment;
	
	if ( window != NULL )
	{
		switch( menuItem )
		{
			case kItemAlignDefault:
				alignment = weFlushDefault;
			break;
			
			case kItemAlignLeft:
				alignment = weFlushLeft;
			break;
			
			case kItemCenter:
				alignment = weCenter;
			break;
			
			case kItemAlignRight:
				alignment = weFlushRight;
			break;
			
			case kItemJustify:
				alignment = weJustify;
			break;
			
			default:
				return;
			break;
		}
		
		// set the alignment mode (this automatically redraws the text)
		
		WESetAlignment( alignment, GetWindowWE(window) );
	}
	
	return;
}


void	DoFeatureChoice( short menuItem )
{
	WindowRef		window = FrontWindow();
	WEReference		we;
	short			feature, oldSetting;
	
	if ( window == NULL )
		return;
	
	we = GetWindowWE(window);
	
	if ( menuItem == kItemTabHooks )
	{
		// install or remove our custom tab hooks
		
		if ( WEIsTabHooks( we ) == false )
		{
			// left-align the text (the hooks only work with left-aligned text )
			WESetAlignment( weFlushLeft, we );
			
			// install tab hooks
			WEInstallTabHooks( we );
		}
		else
		{
			// remove tab hooks
			WERemoveTabHooks( we );
		}
		
		// turn the cursor into a wristwatch
		
		SetCursor( *(GetCursor( watchCursor) ) );
		
		// recalculate link breaks and redraw the text
		
		if ( WECalText( we ) != noErr )
			;
	}
	else
	{
		switch ( menuItem )
		{
			case kItemAutoScroll:
				feature = weFAutoScroll;
			break;
			
			case kItemOutlineHilite:
				feature = weFOutlineHilite;
			break;
			
			case kItemReadOnly:
				feature = weFReadOnly;
			break;
			
			case kItemIntCutAndPaste:
				feature = weFIntCutAndPaste;
			break;
			
			case kItemDragAndDrop:
				feature = weFDragAndDrop;
			break;
			
			case kItemOffscreenDrawing:
				feature = weFDrawOffscreen;
			break;
			
			default:
				;
			break;
		}
		
		// toggle the specified feature
		
		oldSetting = WEFeatureFlag( feature, weBitToggle, we );
	}
	
	return;
}


void	DoMenuChoice( long menuChoice )
{
	short		menuID, menuItem;
	
	// extract menu ID and menu item from menuChoice
	
	menuID = HiWrd( menuChoice );
	menuItem = LoWrd( menuChoice );
	
	// dispatch on menuID
	
	switch( menuID )
	{
		case kMenuApple:
			DoAppleChoice( menuItem );
		break;
		
		case kMenuFile:
			DoFileChoice( menuItem );
		break;
		
		case kMenuEdit:
			DoEditChoice( menuItem );
		break;
		
		case kMenuFont:
			DoFontChoice( menuItem );
		break;
		
		case kMenuSize:
			DoSizeChoice( menuItem );
		break;
		
		case kMenuStyle:
			DoStyleChoice( menuItem );
		break;
		
		case kMenuColor:
			DoColorChoice( menuItem );
		break;
		
		case kMenuFeatures:
			DoFeatureChoice( menuItem );
		break;
		
		case kMenuAlignment:
			DoAlignChoice( menuItem );
		break;
		
		default:
			;
	}
	
	HiliteMenu( 0 );
	
	return;
}


OSErr	InitializeMenus( void )
{
	OSErr		err = noErr;
	
	// build up the whole menu bar from the 'MBAR' resource
	
	SetMenuBar( GetNewMBar( kMenuBarID ) );
	
	// add names to the apple and Font menus
	
	AppendResMenu( GetMenuHandle( kMenuApple ), kTypeDeskAccessory );
	AppendResMenu( GetMenuHandle( kMenuFont ), kTypeFont );
	
	// insert the alignment subment into the hierarchical portion of the menu list
	InsertMenu( GetMenu( kMenuAlignment), -1 );
	
	// disable the "Drag and Drop Editing" item in the Features menu once and for all
	// if the Drag Manager isn't available
	
	if ( gHasDragAndDrop == false )
		DisableItem( GetMenuHandle( kMenuFeatures ), kItemDragAndDrop );
	
	// load the menu color table for the color menu
	
	sColors = (MenuCRsrcHandle)GetResource( kTypeMenuColorTable, kMenuColor );
	err = ResError();
	if ( err != noErr )
		return err;
	HNoPurge( (Handle)sColors );
	
	// draw the menu bar
	
	DrawMenuBar();
	
	return err;
}